import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import argparse
import os

# --- Parse Command-Line Arguments ---
parser = argparse.ArgumentParser(description="Run entropy diffusion simulation.")
parser.add_argument('--init', choices=['global', 'local_spike', 'local_noise'], default='local_spike', help='Initialization mode')
parser.add_argument('--grid', type=int, default=50, help='Grid size (NxN)')
parser.add_argument('--steps', type=int, default=100, help='Number of timesteps')
parser.add_argument('--alpha', type=float, default=0.2, help='Diffusion rate')
parser.add_argument('--decay', type=float, default=0.001, help='Decay rate per timestep')
parser.add_argument('--noise', type=float, default=0.0, help='Noise level for "local_noise" init')
parser.add_argument('--output', type=str, default='entropy_sim', help='Output filename prefix')
args = parser.parse_args()

# --- Simulation Setup ---
grid_size = args.grid
timesteps = args.steps
alpha = args.alpha
decay = args.decay
noise = args.noise
output_prefix = args.output

entropy_grid = np.zeros((timesteps, grid_size, grid_size))

# --- Initialization ---
if args.init == 'global':
    entropy_grid[0] = np.ones((grid_size, grid_size)) * 10
elif args.init == 'local_spike':
    center = grid_size // 2
    entropy_grid[0, center, center] = 100
elif args.init == 'local_noise':
    entropy_grid[0] = np.random.normal(loc=0.0, scale=noise * 100, size=(grid_size, grid_size))

# --- Simulation Loop ---
for t in range(1, timesteps):
    entropy_grid[t] = entropy_grid[t - 1].copy()
    for i in range(1, grid_size - 1):
        for j in range(1, grid_size - 1):
            avg_neighbors = (
                entropy_grid[t - 1, i + 1, j] +
                entropy_grid[t - 1, i - 1, j] +
                entropy_grid[t - 1, i, j + 1] +
                entropy_grid[t - 1, i, j - 1]
            ) / 4
            entropy_grid[t, i, j] += alpha * (avg_neighbors - entropy_grid[t - 1, i, j])
    entropy_grid[t] -= decay  # Apply decay
    entropy_grid[t] = np.clip(entropy_grid[t], 0, None)  # Prevent negative values

# --- Save GIF ---
fig, ax = plt.subplots()
cax = ax.imshow(entropy_grid[0], cmap='inferno', vmin=0, vmax=np.max(entropy_grid[0]))
fig.colorbar(cax)

def update(frame):
    cax.set_data(entropy_grid[frame])
    ax.set_title(f'Timestep {frame}')
    return cax,

ani = animation.FuncAnimation(fig, update, frames=timesteps, blit=False)
gif_path = f"{output_prefix}.gif"
ani.save(gif_path, writer='pillow', fps=10)
plt.close(fig)

# --- Save Entropy Trend Plot ---
total_entropy = [np.sum(entropy_grid[t]) for t in range(timesteps)]
plt.figure(figsize=(8, 4))
plt.plot(range(timesteps), total_entropy, color='blue')
plt.title("Total Entropy Over Time")
plt.xlabel("Timestep")
plt.ylabel("Total Entropy")
plt.tight_layout()
plot_path = f"{output_prefix}_trend.png"
plt.savefig(plot_path)
plt.close()

print(f"Simulation complete. Saved {gif_path} and {plot_path}.")

